@@ -22,6 +22,7 @@
     clippy::use_self,
 )]
 
+mod absolute_path;
 mod app_config;
 mod args;
 mod config;
@@ -53,9 +54,9 @@ use console::style;
 use copy::copy_files_recursively;
 use env_logger::fmt::Formatter;
 use fs_err as fs;
-use hooks::execute_hooks;
+use hooks::{execute_hooks, RhaiHooksContext};
 use ignore_me::remove_dir_files;
-use interactive::prompt_and_check_variable;
+use interactive::{prompt_and_check_variable, LIST_SEP};
 use log::Record;
 use log::{info, warn};
 use project_variables::{StringEntry, StringKind, TemplateSlots, VarInfo};
@@ -434,19 +435,20 @@ fn expand_template(
     args: &GenerateArgs,
 ) -> Result<PathBuf> {
     let liquid_object = create_liquid_object(user_parsed_input)?;
+    let context = RhaiHooksContext {
+        liquid_object: liquid_object.clone(),
+        allow_commands: user_parsed_input.allow_commands(),
+        silent: user_parsed_input.silent(),
+        working_directory: template_dir.to_owned(),
+        destination_directory: user_parsed_input.destination().to_owned(),
+    };
 
     // run init hooks - these won't have access to `crate_name`/`within_cargo_project`
     // variables, as these are not set yet. Furthermore, if `project-name` is set, it is the raw
     // user input!
     // The init hooks are free to set `project-name` (but it will be validated before further
     // use).
-    execute_hooks(
-        template_dir,
-        &liquid_object,
-        &config.get_init_hooks(),
-        user_parsed_input.allow_commands(),
-        user_parsed_input.silent(),
-    )?;
+    execute_hooks(&context, &config.get_init_hooks())?;
 
     let project_name_input = ProjectNameInput::try_from((&liquid_object, user_parsed_input))?;
     let project_name = ProjectName::from((&project_name_input, user_parsed_input));
@@ -488,14 +490,14 @@ fn expand_template(
     )?;
     add_missing_provided_values(&liquid_object, user_parsed_input.template_values())?;
 
+    let context = RhaiHooksContext {
+        liquid_object: Arc::clone(&liquid_object),
+        destination_directory: destination.as_ref().to_owned(),
+        ..context
+    };
+
     // run pre-hooks
-    execute_hooks(
-        template_dir,
-        &liquid_object,
-        &config.get_pre_hooks(),
-        user_parsed_input.allow_commands(),
-        user_parsed_input.silent(),
-    )?;
+    execute_hooks(&context, &config.get_pre_hooks())?;
 
     // walk/evaluate the template
     let all_hook_files = config.get_hook_files();
@@ -512,7 +514,7 @@ fn expand_template(
         user_parsed_input.silent(),
         rhai_filter_files.clone(),
     );
-    template::walk_dir(
+    let result = template::walk_dir(
         &mut template_config,
         template_dir,
         &all_hook_files,
@@ -520,17 +522,24 @@ fn expand_template(
         rhai_engine,
         &rhai_filter_files,
         &mut pbar,
-        args.verbose,
-    )?;
+        args.quiet,
+    );
+
+    match result {
+        Ok(()) => (),
+        Err(e) => {
+            // Don't print the error twice
+            if !args.quiet && args.continue_on_error {
+                warn!("{}", e);
+            }
+            if !args.continue_on_error {
+                return Err(e);
+            }
+        }
+    };
 
     // run post-hooks
-    execute_hooks(
-        template_dir,
-        &liquid_object,
-        &config.get_post_hooks(),
-        user_parsed_input.allow_commands(),
-        user_parsed_input.silent(),
-    )?;
+    execute_hooks(&context, &config.get_post_hooks())?;
 
     // remove all hook and filter files as they are never part of the template output
     let rhai_filter_files = rhai_filter_files
@@ -611,6 +620,27 @@ fn read_default_variable_value_from_template(slot: &TemplateSlots) -> Result<Str
     Ok(default_value)
 }
 
+/// Turn things into strings that can be turned into strings
+/// Tables are not allowed and will be ignored
+/// arrays are allowed but will be flattened like so
+/// [[[[a,b],[[c]]],[[[d]]]]] => "a,b,c,d"
+fn extract_toml_string(value: &toml::Value) -> Option<String> {
+    match value {
+        toml::Value::String(s) => Some(s.clone()),
+        toml::Value::Integer(s) => Some(s.to_string()),
+        toml::Value::Float(s) => Some(s.to_string()),
+        toml::Value::Boolean(s) => Some(s.to_string()),
+        toml::Value::Datetime(s) => Some(s.to_string()),
+        toml::Value::Array(s) => Some(
+            s.iter()
+                .filter_map(extract_toml_string)
+                .collect::<Vec<String>>()
+                .join(LIST_SEP),
+        ),
+        toml::Value::Table(_) => None,
+    }
+}
+
 // Evaluate the configuration, adding defined placeholder variables to the liquid object.
 fn fill_placeholders_and_merge_conditionals(
     config: &mut Config,
@@ -623,14 +653,9 @@ fn fill_placeholders_and_merge_conditionals(
     loop {
         // keep evaluating for placeholder variables as long new ones are added.
         project_variables::fill_project_variables(liquid_object, config, |slot| {
-            let provided_value = template_values.get(&slot.var_name).and_then(|v| match v {
-                toml::Value::String(s) => Some(s.clone()),
-                toml::Value::Integer(s) => Some(s.to_string()),
-                toml::Value::Float(s) => Some(s.to_string()),
-                toml::Value::Boolean(s) => Some(s.to_string()),
-                toml::Value::Datetime(s) => Some(s.to_string()),
-                toml::Value::Array(_) | toml::Value::Table(_) => None,
-            });
+            let provided_value = template_values
+                .get(&slot.var_name)
+                .and_then(extract_toml_string);
             if provided_value.is_none() && args.silent {
                 let default_value = match read_default_variable_value_from_template(slot) {
                     Ok(string) => string,
@@ -748,7 +773,7 @@ impl Drop for ScopedWorkingDirectory {
 #[cfg(test)]
 mod tests {
     use crate::{
-        auto_locate_template_dir,
+        auto_locate_template_dir, extract_toml_string,
         project_variables::{StringKind, VarInfo},
         tmp_dir,
     };
@@ -811,7 +836,7 @@ mod tests {
         let actual = auto_locate_template_dir(tmp.path().to_path_buf(), &mut |slots| match &slots
             .var_info
         {
-            VarInfo::Bool { .. } => anyhow::bail!("Wrong prompt type"),
+            VarInfo::Bool { .. } | VarInfo::Array { .. } => anyhow::bail!("Wrong prompt type"),
             VarInfo::String { entry } => {
                 if let StringKind::Choices(choices) = entry.kind.clone() {
                     let expected = vec!["sub1".to_string(), "sub2".to_string()];
@@ -859,7 +884,7 @@ mod tests {
         let actual = auto_locate_template_dir(tmp.path().to_path_buf(), &mut |slots| match &slots
             .var_info
         {
-            VarInfo::Bool { .. } => anyhow::bail!("Wrong prompt type"),
+            VarInfo::Bool { .. } | VarInfo::Array { .. } => anyhow::bail!("Wrong prompt type"),
             VarInfo::String { entry } => {
                 if let StringKind::Choices(choices) = entry.kind.clone() {
                     let (expected, answer) = match prompt_num {
@@ -904,7 +929,7 @@ mod tests {
         let actual = auto_locate_template_dir(tmp.path().to_path_buf(), &mut |slots| match &slots
             .var_info
         {
-            VarInfo::Bool { .. } => anyhow::bail!("Wrong prompt type"),
+            VarInfo::Bool { .. } | VarInfo::Array { .. } => anyhow::bail!("Wrong prompt type"),
             VarInfo::String { entry } => {
                 if let StringKind::Choices(choices) = entry.kind.clone() {
                     let expected = vec![
@@ -955,4 +980,33 @@ mod tests {
         fs::File::create(&path)?.write_all(contents.as_ref().as_ref())?;
         Ok(())
     }
+
+    #[test]
+    fn test_extract_toml_string() {
+        assert_eq!(
+            extract_toml_string(&toml::Value::Integer(42)),
+            Some(String::from("42"))
+        );
+        assert_eq!(
+            extract_toml_string(&toml::Value::Float(42.0)),
+            Some(String::from("42"))
+        );
+        assert_eq!(
+            extract_toml_string(&toml::Value::Boolean(true)),
+            Some(String::from("true"))
+        );
+        assert_eq!(
+            extract_toml_string(&toml::Value::Array(vec![
+                toml::Value::Integer(1),
+                toml::Value::Array(vec![toml::Value::Array(vec![toml::Value::Integer(2)])]),
+                toml::Value::Integer(3),
+                toml::Value::Integer(4),
+            ])),
+            Some(String::from("1,2,3,4"))
+        );
+        assert_eq!(
+            extract_toml_string(&toml::Value::Table(toml::map::Map::new())),
+            None
+        );
+    }
 }
